/*
******************************************************************************

   LibBuf extension API header file.

   Copyright (c) Wed Aug 29 2001 by: 
	Brian S. Julin		bri@tull.umassp.edu
        Christoph Egger         Christoph_Egger@t-online.de
  
   
This software may be redistributed under the terms of the GNU Library
(a.k.a. "Lesser") Public License -- see the file COPYING in the parent 
directory of the source code.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


******************************************************************************
*/

#ifndef _GGI_BUF_H
#define _GGI_BUF_H

#ifndef _GGI_INTERNAL_GII_H
#include <ggi/ggi.h>
#endif

#include <ggi/galloc.h>

__BEGIN_DECLS

/* Extension management
 */
int ggiBufInit(void);
int ggiBufExit(void);

int ggiBufAttach(ggi_visual_t vis, ggiGA_resource_list *reqlist);
int ggiBufDetach(ggi_visual_t vis);

/* Structure
 */

#ifdef HAVE_STRUCT_BUF
typedef struct ggiBuf *ggiBuf_t;
#else
struct ggiBuf { };

typedef struct ggiBuf *ggiBuf_t;
#endif

typedef ggi_pixel ggi_bufval;


/* Flags

   Some simple flags for people not using carbs/LibBlt directly.
   These are translated to appropriate LibGAlloc flags and/or other settings.

*/

enum ggiBuf_flags {

	BUF_BLEND_OVER		= 0x00000001,
	BUF_BLEND_SRC_OVER	= 0x00000001,
	BUF_BLEND_UNDER		= 0x00000010,
	BUF_BLEND_DST_OVER      = 0x00000010,
	BUF_BLEND_SRC_IN	= 0x00000002,
	BUF_BLEND_DST_IN	= 0x00000020,
	BUF_BLEND_SRC_OUT	= 0x00000004,
	BUF_BLEND_DST_OUT       = 0x00000040,
	BUF_BLEND_TOP		= 0x00000008,
	BUF_BLEND_SRC_TOP	= 0x00000008,
	BUF_BLEND_DST_TOP	= 0x00000080,
	BUF_BLEND_BOTTOM	= 0x00000080,
	BUF_BLEND_XOR		= 0x00000100,
	BUF_BLEND_PLUS		= 0x00000200,

	BUF_BLENDTYPE_MASK	= 0x000003ff,

	BUF_BLEND_POSTDIV	= 0x00000400,
	BUF_DISPOSE		= 0x00000800,

#define BUF_ALTBLEND_SHIFT	16
#define BUF_TOGGLE_SHIFT	16

	BUF_BLEND_ALT		= 0x00001000,

	BUF_PUTGET		= 0x10000000,
	BUF_BLEND_SRC_PREMULT	= 0x40000000,
	BUF_BLEND_DST_PREMULT	= 0x80000000,
	BUF_CMP_UNSIGNED	= 0x00000000,
	BUF_CMP_SIGNED		= 0x20000000,
	BUF_CMP_GT		= 0x00002000,
	BUF_CMP_LT		= 0x00004000,
	BUF_CMP_EQ		= 0x00008000,
	BUF_CMP_GTE		= 0x0000a000,
	BUF_CMP_LTE		= 0x0000c000,
	BUF_CMP_MASK		= 0x0000e000

};	/* ggiBuf_flags */

/* Helper values used internally to map to/from LibGAlloc values */
#define BUF2GA_ZDISPOSE_SHIFT	       1
#define BUF2GA_ALPHA_BLEND_MASK	       0x00000fff
#define BUF2GA_FLAGS_INVALID4ALPHA_VIS 0xf000f000
#define BUF2GA_FLAGS_INVALID4ALPHA     0x3000f000
#define BUF2GA_FLAGS_INVALID4Z         0xd7ff17ff
#define BUF2GA_Z_MASK		       0x2000e000

/* API functions
 */

int ggiBufAdd(ggi_visual_t vis, 
	      ggiGA_resource_list *reqlist,
	      enum ggiGA_resource_type subtype,
	      struct ggiGA_resource_props *props,
	      enum ggiBuf_flags flags,
	      ggiGA_resource_handle tothis,
	      ggiGA_resource_handle *handle);

int ggiBufAdd2Group(ggiGA_resource_list *reqlist,
		    struct ggiGA_resource_props *prop,
		    enum ggiGA_resource_type type,
		    ggiGA_resource_handle *handle);

ggiBuf_t ggiBufTie(ggi_visual_t vis, ggiGA_resource_handle handle_tank,
		   int narg, void *args);

int ggiBufUntie(ggiBuf_t *buf);

ggiGA_resource_handle ggiBufGetMotorHandle(ggiBuf_t buf);
ggiGA_resource_handle ggiBufGetTankHandle(ggiBuf_t buf);
ggiGA_resource_handle ggiBufGetCarbHandle(ggiBuf_t buf);

/* Fills properties with useful default values */
int ggiBufClearMotorProperties(struct ggiGA_resource_props *prop);
int ggiBufClearCarbProperties(struct ggiGA_resource_props *prop);
int ggiBufClearTankProperties(struct ggiGA_resource_props *prop);

/* Group buffers together, e.g. add an alpha channel to a sprite.
 */
int ggiBufClad(ggiBuf_t buf, int numbufs, ggiBuf_t *bufs, ggiBuf_t *res);


/* Use given buffers to overload a visual's drawops.  Will also activate
 * in-pixel alpha channel.
 */
int ggiBufCladVis(ggi_visual_t vis, int numbufs, ggiBuf_t *bufs);


/* Restores to simple GGI visual
 */
int ggiBufStrip(ggi_visual_t vis);

ggiBuf_t ggiBufCreateBuffer(ggi_visual_t vis, 
			    enum ggiGA_resource_type fulltype,
			    struct ggiGA_resource_props *props,
			    enum ggiBuf_flags flags, ggiBuf_t forthis);

ggiBuf_t ggiBufCreateZ(ggi_visual_t vis, ggiBuf_t forthis,
		       enum ggiGA_storage_type stype,
		       ggi_graphtype gt, enum ggiBuf_flags flags);

ggiBuf_t ggiBufCreateAlpha(ggi_visual_t vis, ggiBuf_t forthis,
			   enum ggiGA_storage_type stype,
		           ggi_graphtype gt, enum ggiBuf_flags flags);

/* Destroys the object and releases the resource if it is the
 * last one holding.
 */
int ggiBufDestroyBuffer(ggiBuf_t *buf);




/* Buffer access functions
 */

/* background value is used by ggiBufClear and for the background
 * of ggiPutc/ggiPuts character cells.
 *
 * foreground is used as a constant value for a missing channel,
 * when a mundane primitive is drawing on a clad visual.
 *
 * These values will be automatically initialized to sensible defaults
 * when the buffer is allocated.  (BTW, for in-pixel channels, this
 * would be done by normal (now overloaded) ggiSetGCFGColor).
 */

int ggiBufSetGCForeground(ggiBuf_t buf, ggi_bufval val);
int ggiBufSetGCBackground(ggiBuf_t buf, ggi_bufval val);
int ggiBufSetGCFlags(ggiBuf_t buf, enum ggiBuf_flags flags);

int ggiBufGetGCForeground(ggiBuf_t buf, ggi_bufval *val);
int ggiBufGetGCBackground(ggiBuf_t buf, ggi_bufval *val);
int ggiBufGetGCFlags(ggiBuf_t buf, enum ggiBuf_flags *flags);

/* Set whole buffer to the bg_val */
int ggiBufClearBuffer(ggiBuf_t buf);

int ggiBufBuf2Mem(ggiBuf_t src, size_t xoffset, size_t yoffset, size_t stride,
                  void *dest, size_t deststride,
                  size_t linewidth, size_t numlines);

int ggiBufMem2Buf(ggiBuf_t dest, size_t xoffset, size_t yoffset, size_t stride,
                  void *membuf, size_t srcstride,
                  size_t linewidth, size_t numlines);

int ggiBufBuf2Buf(ggiBuf_t dest, size_t xoffset, size_t yoffset, size_t stride,
                  ggiBuf_t src, size_t srcxoffset, size_t srcyoffset, 
		  size_t srcstride,
                  size_t linewidth, size_t numlines);

int ggiBufConst2Buf(ggiBuf_t dest, size_t xoffset, size_t yoffset, 
		    size_t stride,
                    ggi_bufval val, size_t linewidth, size_t numlines);

/* Return values for above */
#define BUF_ACCESSFUNC_UNNEEDED GGI_ACCESSFUNC_UNNEEDED
#define BUF_ACCESSFUNC_NOFLUSH  GGI_ACCESSFUNC_NOFLUSH

/* User convenience macros for 1D buffers
 */
#define ggiBufBuf2Mem1D(src, offset, dest, linewidth)           \
        ggiBufBuf2Mem(src, offset, 0, 0, dest, 0, linewidth, 0);

#define ggiBufMem2Buf1D(dest, offset, membuf, linewidth)        \
        ggiBufMem2Buf(dest, offset, 0, 0, membuf, 0, linewidth, 0);

#define ggiBufBuf2Buf1D(dest, offset, src, srcoffset, srcstride, linewidth)  \
	ggiBufBuf2Buf(dest, offset, 0, 0, src, srcoffset, 0,    \
			0, srcstride, linewidth, 0);

#define ggiBufConst2Buf1D(dest, offset, val, linewidth)         \
        ggiBufConst2Buf(dest, offset, 0, 0, val, linewidth, 0);


/* User convenient helper functions */
int ggiBufGetValue(ggiBuf_t buf, size_t xoffset, size_t yoffset,
                   size_t stride, ggi_bufval *val);
int ggiBufPutValue(ggiBuf_t buf, size_t xoffset, size_t yoffset,
                   size_t stride, ggi_bufval val);

#define ggiBufGetValue1D(buf, offset, val)      \
        ggiBufGetValue(buf, offset, 0, 0, val);

#define ggiBufPutValue1D(buf, offset, val)      \
        ggiBufPutValue(buf, offset, 0, 0, val);


__END_DECLS

#endif /* _GGI_BUF_H */

