/*
******************************************************************************

   LibBuf extension internals.

   Copyright (c) Wed Aug 29 2001 by: 
	Brian S. Julin		bri@tull.umassp.edu
        Christoph Egger         Christoph_Egger@t-online.de
  
   
This software may be redistributed under the terms of the GNU Library
(a.k.a. "Lesser") Public License -- see the file COPYING in the parent 
directory of the source code.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


******************************************************************************
*/

#ifndef _GGI_BUF_INT_H
#define _GGI_BUF_INT_H

#include <ggi/internal/lowlevel.h>

#define HAVE_STRUCT_BUF
#include <ggi/buf.h>

__BEGIN_DECLS
/* Exported variables */
#ifdef BUILDING_LIBBUF
extern ggi_extid    _BufID;
#else
IMPORTVAR ggi_extid _BufID;
#endif

__END_DECLS

/* Our lovely getapi suggest string munger */
#define BUF_GETAPI_GIVE_PIXFMT 1
#define BUF_GETAPI_ALPHA_USED  2
void BUF_getapi_fstr (ggi_visual *vis, char* str, int flags);


/* One of these is to be kept in the clad visual's private area for each
 * attached buffer, and compared to the values in the object itself before 
 * drawops so the hardware state can be adjusted if needed. That part 
 * isn't implemented in this tree.
 */

struct ggi_bufgc {
	ggi_bufval fg_val, bg_val;
	enum ggiBuf_flags flags;
};

struct ggiBuf {

	/* These can be shared with other ggiBuf_t's */
        ggi_ll_storage_t     storage;
	ggi_ll_storage_ops_t storage_ops;  
	ggi_directbuffer *db;      

	/* The rest are all per-buffer */
	int w, h; 
	struct ggi_bufgc gc;

	/* GA_TYPE of the resource, or 0 if a clad buffer. */
	enum ggiGA_resource_type type; 
	union {
		struct { ggiGA_resource_handle tank, motor, carb; } res;
		struct { ggiBuf_t *bufs; int numbufs; } clad;
	} parts;

	/* Convenience, to keep from having to dig into parts.res */
	struct ggiGA_resource_props *tank_p, *carb_p;

	ggi_visual_t vis; /* Visual we were created for */

};


/* These are per-visual overloads but not visible in public API */
int ggiBuf_untie(ggi_visual *vis, ggiBuf_t *buf);
int ggiBuf_clad(ggi_visual *vis, int flags, int numbufs, 
		ggiBuf_t *bufs, ggiBuf_t *res);

/* Values for ggiBuf_clad's flags parameter */
#define BUF_CLAD_VISUAL		0x0001

/* Exported variables */
#ifdef BUILDING_LIBBUF
extern ggi_extid    _BufID;
#else
IMPORTVAR ggi_extid _BufID;
#endif

__END_DECLS

/*-* Data types for the API functions */
typedef int (Buffunc_ggiBufStrip)(ggi_visual *vis);
typedef int (Buffunc_ggiBuf_untie)(ggi_visual *vis,
				ggiBuf_t *buf);
typedef int (Buffunc_ggiBuf_clad)(ggi_visual *vis,
				int flags, int numbufs,
				ggiBuf_t *bufs, ggiBuf_t *res);
typedef ggiBuf_t (Buffunc_ggiBuf_tie)(ggi_visual_t vis, 
				      ggiGA_resource_handle handle_tank,
				      ggiGA_resource_handle handle_motor,
				      ggiGA_resource_handle handle_carb, 
				      int narg, void *args);

typedef struct bufpriv {
	/*-* API function pointers */
	Buffunc_ggiBufStrip	*strip;
	Buffunc_ggiBuf_untie	*_untie;
	Buffunc_ggiBuf_clad	*_clad;
	Buffunc_ggiBuf_tie	*_tie;


	/* pointer is needed to override libggi's getapi function */
	ggifunc_getapi		*getapi;
	void 			*cfghand; /* Hack for storage sublibs */

	struct ggi_ll_storage_ops def_ops;

	/*-* Other structure members */
	int is_clad;
	struct ggi_visual_opdraw * old_opdraw;

	/* We are clad with this Alpha buffer */
	ggiBuf_t abuf;	
	int achannel;
	struct ggi_bufgc abuf_gc; /* hw state shadowed here. */

	/* We are clad with this Z buffer */
	ggiBuf_t zbuf;
	struct ggi_bufgc zbuf_gc; /* hw state shadowed here */


	ggiGA_resource_list *reqlist;  /* Pointer to application provided
                                          "global" request list for lazy API
                                        */

	void *priv;	/* Here sublibs can store visual specific private data.
			 * If you have more than one sublib (for each visual)
			 * that needs private data, just add more pointers or
			 * add the data fields directly in this structure.
			 */
} bufpriv;


/* Used to access the bufpriv structure for a particular visual. */
#define LIBGGI_BUFEXT(vis) ((bufpriv *)LIBGGI_EXT((vis),_BufID))



#endif /* _GGI_BUF_INT_H */
